"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var abort_controller_1 = require("abort-controller");
var chalk_1 = require("chalk");
var ora = require("ora");
var split = require("split");
var yargs = require("yargs");
var complaints_1 = require("./complaints");
var git_1 = require("./git");
var lint_blame_1 = require("./lint-blame");
var argv = yargs
    .usage('Usage: <linter> | blame-lint [options]')
    .version(require('../package.json').version)
    .default('config', './lint-blame.json')
    .config('config')
    .alias('c', 'config')
    .env('LINT_BLAME')
    .help()
    .option('members', {
    type: 'array',
    description: 'A list of { email, name } members that rules should apply for',
})
    .option('format', {
    alias: 'f',
    description: 'The complaint format to parse',
    choices: ['tslint4', 'tslint5', 'tsconfig'],
    required: true,
})
    .option('since', {
    type: 'string',
    coerce: function (since) { return new Date(since); },
    description: 'A point in time before which rules do not apply',
}).argv;
if (!argv.file && process.stdin.isTTY) {
    throw new Error('No input on STDIN');
}
var parseComplaint = complaints_1.parsers[argv.format];
if (!parseComplaint) {
    throw new Error("Unknown complaint format: " + argv.format);
}
var formatBlame = function (blame) {
    if (!blame) {
        return chalk_1.default.red('Not Committed Yet');
    }
    var author = blame.author || 'No Author';
    var date = (blame.authorTime && blame.authorTime.toLocaleString()) || 'No Author Date';
    return chalk_1.default.yellow(blame.sha1.slice(0, 7)) + " " + chalk_1.default.cyan(author) + " " + chalk_1.default.magenta(date);
};
process.on('uncaughtException', function (err) {
    throw err;
});
process.on('unhandledRejection', function (err) {
    throw err;
});
var blamer = new git_1.Blamer(20);
var totalComplaints = 0;
var validComplaints = 0;
var abortController = new abort_controller_1.AbortController();
process.on('SIGTERM', function () { return abortController.abort(); });
process.on('exit', function () { return abortController.abort(); });
var spinner = ora('waiting for input on STDIN').start();
// Read lines from STDIN
var lineStream = process.stdin.pipe(split());
var promises = [];
lineStream.on('data', function (line) {
    promises.push((function () { return __awaiter(_this, void 0, void 0, function () {
        var complaint, blame;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    line = line.trim();
                    spinner.text = line;
                    if (!line) {
                        return [2 /*return*/];
                    }
                    try {
                        complaint = parseComplaint(line + '');
                    }
                    catch (_b) {
                        return [2 /*return*/];
                    }
                    totalComplaints++;
                    return [4 /*yield*/, blamer.blameLine(complaint.filePath, complaint.line, abortController.signal)];
                case 1:
                    blame = _a.sent();
                    if (!blame || !lint_blame_1.checkBlame(blame, argv)) {
                        // Ignore complaint
                        return [2 /*return*/];
                    }
                    spinner.stop();
                    process.stdout.write(formatBlame(blame) + " " + line + "\n");
                    spinner.start();
                    validComplaints++;
                    process.exitCode = 1;
                    return [2 /*return*/];
            }
        });
    }); })());
});
lineStream.on('end', function () { return __awaiter(_this, void 0, void 0, function () {
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, Promise.all(promises)];
            case 1:
                _a.sent();
                spinner.stop();
                process.stdout.write("\n" + validComplaints + " complaints (" + totalComplaints + " total, " + (totalComplaints - validComplaints) + " ignored)\n");
                return [2 /*return*/];
        }
    });
}); });
//# sourceMappingURL=cli.js.map