"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var abort_controller_1 = require("abort-controller");
var child_process_1 = require("child_process");
var semaphore_async_await_1 = require("semaphore-async-await");
/**
 * Parses and sets data from a line following a commit header
 *
 * @param {array} lineArr The current line split by a space
 */
function parseCommitLine(line) {
    var spaceIndex = line.indexOf(' ');
    var key = line.slice(0, spaceIndex);
    var value = line.slice(spaceIndex + 1);
    switch (key) {
        case 'author':
            return { author: value };
        case 'author-mail':
            return { authorMail: value };
        case 'author-time':
            return { authorTime: new Date(~~value * 1000) };
        case 'author-tz':
            return { authorTz: value };
        case 'committer':
            return { committer: value };
        case 'committer-mail':
            return { committerMail: value };
        case 'committer-time':
            return { committerTime: new Date(~~value * 1000) };
        case 'committer-tz':
            return { committerTz: value };
        case 'summary':
            return { summary: value };
        case 'filename':
            return { filename: value };
        case 'previous':
            return { previousHash: value };
    }
    return {};
}
var NOT_COMMITED_YET_SHA1 = '0'.repeat(40);
var BlameParser = /** @class */ (function () {
    function BlameParser() {
        this.commitData = {};
        this.lineData = {};
        this.settingCommitData = false;
        this.currentLineNumber = 1;
    }
    BlameParser.prototype.parse = function (blame) {
        // Split up the original document into an array of lines
        var lines = blame.split('\n');
        // Go through each line
        for (var _i = 0, lines_1 = lines; _i < lines_1.length; _i++) {
            var line = lines_1[_i];
            // If we detect a tab character we know it's a line of code
            // So we can reset stateful variables
            if (line[0] === '\t') {
                // The first tab is an addition made by git, so get rid of it
                this.lineData[this.currentLineNumber].code = line.substr(1);
                this.settingCommitData = false;
                this.currentCommitHash = undefined;
            }
            else {
                // If we are in the process of collecting data about a commit summary
                if (this.settingCommitData) {
                    if (this.currentCommitHash) {
                        Object.assign(this.commitData[this.currentCommitHash], parseCommitLine(line));
                    }
                }
                else {
                    var _a = line.split(' '), sha1 = _a[0], originalLineStr = _a[1], finalLineStr = _a[2], numLinesStr = _a[3];
                    if (sha1 !== NOT_COMMITED_YET_SHA1) {
                        this.currentCommitHash = sha1;
                    }
                    this.currentLineNumber = ~~finalLineStr;
                    // Since the commit data (author, committer, summary, etc) only
                    // appear once in a porcelain output for every commit, we set
                    // it up once here and then expect that the next 8-11 lines of
                    // the file are dedicated to that data
                    if (!this.commitData[sha1]) {
                        this.settingCommitData = true;
                        if (sha1 !== NOT_COMMITED_YET_SHA1) {
                            this.commitData[sha1] = { sha1: sha1 };
                        }
                    }
                    // Setup the new lineData hash
                    this.lineData[this.currentLineNumber] = {
                        code: '',
                        originalLine: ~~originalLineStr,
                        finalLine: ~~finalLineStr,
                        numLines: numLinesStr ? ~~numLinesStr : -1,
                        commit: sha1 !== NOT_COMMITED_YET_SHA1 ? this.commitData[sha1] : null,
                    };
                }
            }
        }
    };
    return BlameParser;
}());
var Blamer = /** @class */ (function () {
    function Blamer(concurrencyLimit) {
        /** Map from absolute file path to pending blame result */
        this.blames = new Map();
        this.concurrencyLimit = new semaphore_async_await_1.default(concurrencyLimit);
    }
    Blamer.prototype.blameLine = function (file, line, signal) {
        if (signal === void 0) { signal = new abort_controller_1.AbortController().signal; }
        return __awaiter(this, void 0, void 0, function () {
            var blames, blame;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.memoizedBlameFile(file, signal)
                        // Linters can report an error on the line that contains EOL, but git blame can't blame it
                    ];
                    case 1:
                        blames = _a.sent();
                        blame = blames && blames[line];
                        return [2 /*return*/, blame && blame.commit];
                }
            });
        });
    };
    Blamer.prototype.memoizedBlameFile = function (file, signal) {
        if (signal === void 0) { signal = new abort_controller_1.AbortController().signal; }
        var promise = this.blames.get(file);
        if (!promise) {
            promise = this.blameFile(file, signal);
            this.blames.set(file, promise);
        }
        return promise;
    };
    Blamer.prototype.blameFile = function (file, signal) {
        if (signal === void 0) { signal = new abort_controller_1.AbortController().signal; }
        return __awaiter(this, void 0, void 0, function () {
            var stdout, err_1, blameParser;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.concurrencyLimit.wait()];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, 4, , 5]);
                        return [4 /*yield*/, new Promise(function (resolve, reject) {
                                var child = child_process_1.execFile('git', ['blame', '--porcelain', file], { encoding: 'utf-8', maxBuffer: Infinity }, function (err, stdout) { return (err ? reject(err) : resolve(stdout)); });
                                signal.addEventListener('abort', function () {
                                    child.kill();
                                    reject(Object.assign(new Error('Aborted'), { name: 'AbortError' }));
                                });
                            })];
                    case 3:
                        stdout = _a.sent();
                        return [3 /*break*/, 5];
                    case 4:
                        err_1 = _a.sent();
                        // If the path is not known to git, the file is not committed yet
                        if (err_1.stderr && err_1.stderr.includes('no such path')) {
                            return [2 /*return*/, null];
                        }
                        throw err_1;
                    case 5:
                        this.concurrencyLimit.signal();
                        blameParser = new BlameParser();
                        blameParser.parse(stdout);
                        return [2 /*return*/, blameParser.lineData];
                }
            });
        });
    };
    return Blamer;
}());
exports.Blamer = Blamer;
//# sourceMappingURL=git.js.map